{% extends "leprikon/default.html" %}
{% load i18n %}

{% block title %}{% trans 'Password reset complete' %}{% endblock %}

{% block content %}

<h1>{% trans 'Password reset complete' %}</h1>

<p>{% trans "Your password has been set.  You may go ahead and log in now." %}</p>

<p><a href="{% url 'leprikon:user_login' %}">{% trans 'Log in' %}</a></p>

{% endblock %}
