{% load i18n %}

{{ object.text_registration_received }}

{% if object.activity.over_capapcity %}
<p>
{% blocktrans with activity_type=object.activity.activity_type.name_genitiv %}Unfortunately, the capacity of this {{ activity_type }} has already been filled.
The registration may be refused.{% endblocktrans %}
</p>
{% endif %}

{% include 'leprikon/registration_mail_details.html' %}

{% include 'leprikon/registration_mail_footer.html' %}
