# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-05-13 07:10
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_pays', '0001_initial'),
        ('leprikon', '0025_agreements'),
    ]

    operations = [
        migrations.AddField(
            model_name='leprikonsite',
            name='payment_gateway',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='django_pays.Gateway', verbose_name='Payment Gateway'),
        ),
        migrations.AddField(
            model_name='subjectpayment',
            name='pays_payment',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='django_pays.Payment', verbose_name='online payment'),
        ),
    ]
