{% extends 'leprikon/default.html' %}
{% load i18n leprikon_tags %}

{% block content %}

<h1>{% block title %}{% blocktrans with activity_type=orderable.activity_type.name_akuzativ name=orderable.name %}Registrations for {{ activity_type }} {{ name }}{% endblocktrans %}{% endblock %}</h1>

<h2>{% trans 'Active Registrations' %}</h2>

{% for reg in orderable.all_approved_registrations %}
{% include 'leprikon/registration_details.html' with reg=reg.orderableregistration %}
{% empty %}
<p>{% blocktrans with activity_type=orderable.activity_type.name_akuzativ %}There are no active registrations for this {{ activity_type }}.{% endblocktrans %}</p>
{% endfor %}

{% if orderable.all_unapproved_registrations %}
    <h2>{% trans 'Unapproved Registrations' %}</h2>

    {% for reg in orderable.all_unapproved_registrations %}
    {% include 'leprikon/registration_details.html' with reg=reg.orderableregistration %}
    {% endfor %}
{% endif %}

{% if orderable.all_inactive_registrations %}
    <h2>{% trans 'Inactive Registrations' %}</h2>

    {% for reg in orderable.all_inactive_registrations %}
    {% include 'leprikon/registration_details.html' with reg=reg.orderableregistration %}
    {% endfor %}
{% endif %}

{% endblock %}
