# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-06-21 13:07
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0053_registration_dates_by'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='subjectregistration',
            name='cancel_request',
        ),
    ]
