# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-05-06 10:37
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


def set_mail_sent(apps, schema_editor):
    SubjectPayment = apps.get_model('leprikon', 'SubjectPayment')
    SubjectPayment.objects.update(mail_sent=models.F('accounted'))


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0051_registration_group_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='subjectpayment',
            name='mail_sent',
            field=models.DateTimeField(editable=False, null=True, verbose_name='mail sent'),
        ),
        migrations.RunPython(set_mail_sent),
    ]
