{% load sekizai_tags static %}

{% addtoblock 'css' %}
<link rel="stylesheet" href="{% static 'leprikon/lib/ekko-lightbox.min.css' %}"/>
{% endaddtoblock %}

{% addtoblock 'js' %}
<script src="{% static 'leprikon/lib/ekko-lightbox.min.js' %}"></script>
{% endaddtoblock %}
