# Generated by Django 1.11.13 on 2018-09-04 18:26

import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
from django.utils.timezone import utc
import djangocms_text_ckeditor.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('leprikon', '0020_agreements'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserAgreement',
            fields=[
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, related_name='agreement', serialize=False, to=settings.AUTH_USER_MODEL, verbose_name='user')),
                ('granted', models.DateTimeField(auto_now=True, verbose_name='time of agreement')),
            ],
        ),
        migrations.RenameField(
            model_name='leprikonsite',
            old_name='agreement',
            new_name='registration_agreement',
        ),
        migrations.AddField(
            model_name='leprikonsite',
            name='user_agreement',
            field=djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', verbose_name='user agreement'),
        ),
        migrations.AddField(
            model_name='leprikonsite',
            name='user_agreement_changed',
            field=models.DateTimeField(blank=True, null=True, verbose_name='last time user agreement changed'),
            preserve_default=False,
        ),
    ]
